## Primitive Types

| Cairo              | TypeScript          |
| ------------------ | ------------------- |
| `felt252`          | `bigint`            |
| `function`         | `number`            |
| `u8 - u32`         | `number \| bigint`  |
| `u64 - u256`       | `bigint`            |
| `ContractAddress`  | `bigint`            |
| `()`               | `void`              |
|  `bool`            | `boolean`           |

##  Complex Types

| Cairo                     | TypeScript                                                                                |
| ------------------------- | ----------------------------------------------------------------------------------------- |
| `Option<T>`               | `T \| undefined`                                                                          |
| `Array<T>`                | `T[]`                                                                                     |
| `tuple (T1, T2, ..., Tn)` | `[T1, T2, ..., Tn]`                                                                       |
| `ContractAddress`         | `bigint`                                                                                  |
| `struct`                  | an object where keys are struct member names and types are inferred using the above rules |
| `enum`                    | a union of enum variants                                                                  |

#### Struct example

**Cairo:**

```cairo
struct TestStruct {
  int128: u128,
  felt: felt252,
  tuple: (u32, u32)
}
```

**Typescript:**

```typescript
{
  int128: bigint;
  felt: bigint;
  tuple: [number | bigint, number | bigint];
}
```

#### Enum example

**Cairo:**

```cairo
enum TestEnum {
  int128: u128,
  felt: felt252,
  tuple: (u32, u32),
}
```

**Typescript:**

```typescript
{ int128: bigint } |
{ felt: bigint } |
{ tuple: [number | bigint, number | bigint]}
```
